<?php

namespace Teknisa\MAN\Services;

use Doctrine\DBAL\Connection;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\NovoCodigo;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Request;
use \Setforms2\Shared\ORM\DateTime;

use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Model\Geral\Entities\Empresa;
use Teknisa\Model\Geral\Entities\Fornecedor;
use Teknisa\Model\Financeiro\Entities\Tipoctpagar;



class ManutencaoCaixaService {

    protected $conn;
    protected $entityManager;
    protected $environment;
    protected $newCode;

    public function __construct(Connection $conn, EntityManager $entityManager,  Environment $environment, NewCode $newCode) {
        $this->conn    = $conn;
        $this->entityManager = $entityManager;
        $this->environment  = $environment;
        $this->newCode = $newCode;
    }

    public function getNrSequenciaMovimentacao($cdcontadorMovcaixa,$NrOrg,$size=10){
        return NovoCodigo::geraCodigo($this->conn,$cdcontadorMovcaixa,$NrOrg,10);
    }
    
    public function saveMovCaixa($request){
        $row = $request->getDataSet()->getRows();
        $currentDate = new \DateTime();
        $params = array(
            ':CDFILIAL' =>  $row[0]->get('CDFILIAL'),
            ':CDCAIXA'  =>  $row[0]->get('CDCAIXA'),
            ':DTABERCAIX'   =>  $row[0]->get('DTABERCAIX'),
            ':NRSEQUMOVI'   =>  $this->getNrSequenciaMovimentacao( 'MOVCAIXA'.$row[0]->get('CDFILIAL').$row[0]->get('CDCAIXA').$row[0]->get('DTABERCAIX'),$row[0]->get('NRORG'),10),
            ':IDTIPOMOVIVE' =>  'A',
            ':DSMOVIVEND'   =>  $row[0]->get('DSOBSCAIXA'),
            ':VRMOVIVEND'   =>  $row[0]->get('VRMOVIVEND'),
            ':CDTIPORECE'   =>  $row[0]->get('CDTIPORECE'),
            ':DTHRINCMOV'   =>  $row[0]->get('DTABERCAIX'),
            ':NRCONTRAVALE' =>  '',
            ':DTMOVIMCAIXA' =>  $currentDate->format("d/m/Y"),
            ':NRORG'    =>  $row[0]->get('NRORG'),
            ':NRORGINCLUSAO'    =>  $row[0]->get('NRORG'),
            ':NRORGULTATU'  =>  $row[0]->get('NRORG'),
            ':CDOPERINCLUSAO'   =>  $row[0]->get('CDOPERADOR'),
            ':CDOPERULTATU' =>  $row[0]->get('CDOPERADOR')
        );
        $this->conn->executeQuery(Utilities\Query::INSERT_MOVCAIXA_ABERTURA, $params);
        
        return $params[':NRSEQUMOVI'];
    }
    
    public function isRegisterOpen($CDFILIAL,$CDCAIXA,$NRORG){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCAIXA'  => $CDCAIXA,
            ':NRORG'    => $NRORG
        );
        $result = $this->conn->fetchAll(Utilities\Query::IS_REGISTER_OPEN,$params);
        if(empty($result)){
            return false;
        } else {
            return true;
        }
    }
    
    public function verifyDateParam ($CDFILIAL,$NRORG){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRORG'    => $NRORG
        );
        $result = $this->conn->fetchAll(Utilities\Query::DATE_PARAM,$params);
        if(empty($result)){
            return false;
        } else {
            return $result[0];
        }
    }
    
    public function atualizaSaldoCaixa ($CDFILIAL,$CDCAIXA,$DTMOVSALDCXA,$CDOPERADOR,$DTATUSALDCXA,$NRORG){
        $params = array(
            ':P_CDFILIAL' => $CDFILIAL,
            ':P_CDCAIXA' => $CDCAIXA,
            ':P_DTMOVSALDCXA' => $DTMOVSALDCXA,
            ':P_NRORG'       => $NRORG
        );
        $DTMOVSALDCXA = DateTime::createFromFormat('d/m/Y', $DTMOVSALDCXA);
        $DTATUSALDCXA = DateTime::createFromFormat('d/m/Y', $DTATUSALDCXA);
        $procedure = null;
        
        $procedure = new StoredProcedure($this->conn, 'SALDO_CAIXA');
        $procedure->addParam(new Param('P_CDFILIAL'  ,     Param::PARAM_INPUT, $CDFILIAL  ,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCAIXA' ,       Param::PARAM_INPUT, $CDCAIXA,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTMOVSALDCXA',   Param::PARAM_INPUT, $DTMOVSALDCXA->format('d/m/Y'),   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',     Param::PARAM_INPUT, $CDOPERADOR,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTATUSALDCXA'  , Param::PARAM_INPUT, $DTATUSALDCXA->format('d/m/Y'),   Param::PARAM_TYPE_STR));
        $procedure->execute();
        $result = $this->conn->fetchAll(self::SALDO_CAIXA,$params);
        if(empty($result)){
            return false;
        } else {
            return $result;
        }
    }
    
    public function checkDataProc ($CDFILIAL, $NRORG, $DTABERTURA){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRORG'       => $NRORG
        );
        $retorno = true;
        $DTABERTURA = explode(' ', $DTABERTURA);
        $DTABERTURA = str_replace('/','-', $DTABERTURA[0]);
        $result = $this->conn->fetchAssoc(self::CHECK_DTPROC , $params);
        if(strtotime($DTABERTURA) > strtotime($result['DTPROCVND'])){
            $DATE = date("d-m-Y", strtotime($result['DTPROCVND']." + ".$result['NRDIAPOSLANC']." day"));
            $retorno = !(strtotime($DATE) < strtotime($DTABERTURA));
            
        } else if (strtotime($DTABERTURA) < strtotime($result['DTPROCVND'])){
            $DATE = date("d-m-Y", strtotime($result['DTPROCVND']." - ".$result['NRDIAANTLANC']." day"));
            $retorno = !(strtotime($DATE) > strtotime($DTABERTURA));
        }
        return $retorno;
    }
    
    public function movCaixa ($CDFILIAL, $NRORG, $DTABERTURA, $CDCAIXA){
        $params = array(
            ':CDFILIAL'  => $CDFILIAL,
            ':NRORG'     => $NRORG,
            ':DTABERCAIX'=> $DTABERTURA,
            ':CDCAIXA'   => $CDCAIXA
        );
        $result = $this->conn->fetchAll(self::MOVCAIXA , $params);
        return $result;
    }
    
    public function verifTurCaixaAux ($CDFILIAL, $NRORG, $DTABERTURA, $CDCAIXA){
        $params = array(
            ':CDFILIAL'  => $CDFILIAL,
            ':NRORG'     => $NRORG,
            ':DTABERCAIX'=> $DTABERTURA,
            ':CDCAIXA'   => $CDCAIXA
        );
        $result = $this->conn->fetchAll(self::VERIFTURCAIXAAUX , $params);
        return $result;
    }
    
    public function transferencia ($CDFILIAL, $NRORG, $DTABERTURA, $CDCAIXA, $MOVCAIXA, $CDCAIXACONT, $CDOPERADOR, $DTMOVTURCAIX){
        $dataAtual = $this->conn->fetchAssoc(self::SELECT_SYSDATE);
        $params = array(
            ':CDFILIAL'   => $CDFILIAL,
            ':NRORG'      => $NRORG,
            ':DTABERCAIX' => $DTABERTURA,
            ':CDCAIXA'    => $CDCAIXA,
            ':CDCAIXACONT'=> $CDCAIXACONT,
            ':CDOPERADOR' => $CDOPERADOR,
            ':DATAATUAL'  => $dataAtual['DATAATUAL'],
            ':DTMOVTURCAIX'=> $DTMOVTURCAIX
        );
        $this->conn->executeQuery(self::INSERT_TURCAIXA , $params);
        $NRSEQUMOVI = $this->conn->fetchAssoc(self::GET_NRSEQUMOVI , $params);
        foreach ($MOVCAIXA as $value) {
            $NRSEQUMOVI['NRSEQUMOVI'] = $NRSEQUMOVI['NRSEQUMOVI'] + 1;
            $params[':IDTIPOMOVIVE'] = $value['TIPO'];
            $params[':VRMOVIVEND']   = $value['VRMOVIVEND'];
            $params[':CDTIPORECE']   = $value['CDTIPORECE'];
            $params[':DTDEPCHEQUE']  = $value['DTDEPCHEQUE'];
            $params[':DTMOVIMCAIXA'] = $value['DTMOVIMCAIXA'];
            $params[':NRENVELOPE']   = $value['NRENVELOPE'];
            $params[':VRMOVIVEOUT']  = $value['VRMOVIVEOUT'];
            $params[':VRCOTINDOUT']  = $value['VRCOTINDOUT'];
            $params[':NRSEQUMOVI']   = $NRSEQUMOVI['NRSEQUMOVI'];
            $this->conn->executeQuery(self::INSERT_MOVCAIXA , $params);
        }
        return true;
    }
    
    public function cancelaTransferencia ($CDFILIAL, $CDCAIXA, $DTABERTURA, $NRORG,  $CDCAIXAORIG  ){
        $params = array(
            ':CDFILIAL'  => $CDFILIAL,
            ':NRORG'     => $NRORG,
            ':DTABERCAIX'=> $DTABERTURA,
            ':CDCAIXA'   => $CDCAIXA,
            ':CDCAIXAORIG'   => $CDCAIXAORIG
        );
        $DTABERTURAORIG = $this->conn->fetchAssoc(self::GET_DTABERCAIX_CONTROLE , $params);
        $params[':DTABERCAIXCONT'] = $DTABERTURAORIG['DTABERCAIX'];
        $this->conn->executeQuery(self::DELETE_MOVCAIXA , $params);
        $this->conn->executeQuery(self::DELETE_TURCAIXA , $params);
        return true;
    }
    
    public function vendasIrregulares($params) {
        
        try {
            
            $result = $this->conn->fetchAll(self::GET_VENDAS_IRREGULARES, $params);
            
        } catch (\Exception $e) {
            throw new \Exception('Ocorreu um erro ao buscar as vendas irregulares.');
        }
        
        return $result;
    }
    
    public function executarQuery($query, $parametros) {
        $this->conn->executeQuery($query, $parametros);
    }
    
    public function nrSeqVendaInvalido($parametros) {
        $result = false;
        
        $retorno = $this->conn->fetchAll(self::NRSEQVENDA_VALIDO, $parametros);
        
        if (sizeof($retorno) == 0) {
            $result = true;
        }
        
        return $result;
    }
    
    const CHECK_DTPROC = "
    SELECT TO_CHAR(DTPROCVND, 'DD-MM-YYYY') AS DTPROCVND, IDPERLANCANTDP, IDPERLANCPOSDP, NRDIAANTLANC, NRDIAPOSLANC
        FROM PARAVEND
        WHERE CDFILIAL = :CDFILIAL AND
              NRORG    = :NRORG AND
        CDPARAVEND ='1' ";
    
    const MOVCAIXA = "
        SELECT A.TIPO, A.CDTIPORECE, A.DTDEPCHEQUE, A.DTMOVIMCAIXA, A.NRENVELOPE,
               SUM(A.VRMOVIVEND) AS VRMOVIVEND, A.VRMOVIVEOUT, A.VRCOTINDOUT
          FROM
          (SELECT 'E' AS TIPO, M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE,
                  M.VRMOVIVEND, M.VRMOVIVEOUT, M.VRCOTINDOUT
             FROM MOVCAIXA M, TIPORECE T
            WHERE (M.CDFILIAL = :CDFILIAL)
              AND (M.CDCAIXA = :CDCAIXA)
              AND (M.NRORG = :NRORG)
              AND (T.NRORG = M.NRORG)
              AND (TO_CHAR(M.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX)
              AND (M.IDTIPOMOVIVE = 'G')
              AND (T.CDTIPORECE = M.CDTIPORECE)
              AND (T.IDTIPORECE <> '7')
           UNION ALL
           SELECT 'S' AS TIPO, M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE,
                  M.VRMOVIVEND, M.VRMOVIVEOUT, M.VRCOTINDOUT
           FROM MOVCAIXA M, TIPORECE T
           WHERE (M.CDFILIAL     = :CDFILIAL) AND
                 (M.CDCAIXA      = :CDCAIXA)  AND
                 (M.NRORG = :NRORG) AND
                 (T.NRORG = M.NRORG) AND
                 (TO_CHAR(M.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS')   = :DTABERCAIX) AND
                 (M.IDTIPOMOVIVE = 'G'          ) AND
                 (T.CDTIPORECE   = M.CDTIPORECE ) AND
                 (T.IDSANGRIAAUTO = 'S'         ) AND
                 (T.IDTIPORECE   <> '7')
           UNION ALL
           SELECT 'S' AS TIPO, M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE,
                  M.VRMOVIVEND, M.VRMOVIVEOUT, M.VRCOTINDOUT
             FROM MOVCAIXA M, TIPORECE T
            WHERE (M.CDFILIAL = :CDFILIAL)
              AND (M.CDCAIXA = :CDCAIXA)
              AND (M.NRORG = :NRORG)
              AND (T.NRORG = M.NRORG)
              AND (TO_CHAR(M.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX)
              AND (M.IDTIPOMOVIVE = 'G')
              AND (T.CDTIPORECE = M.CDTIPORECE)
              AND (T.IDTIPORECE = '7')
           UNION ALL
           SELECT 'E' AS TIPO, M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE,
                  M.VRMOVIVEND, M.VRMOVIVEOUT, M.VRCOTINDOUT
             FROM MOVCAIXA M, TIPORECE T
            WHERE (M.CDFILIAL = :CDFILIAL)
              AND (M.CDCAIXA = :CDCAIXA)
              AND (M.NRORG = :NRORG)
              AND (T.NRORG = M.NRORG)
              AND (TO_CHAR(M.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX)
              AND (M.IDTIPOMOVIVE = 'S')
              AND (T.CDTIPORECE = M.CDTIPORECE)
              AND (T.IDTIPORECE = '7')
           UNION ALL
           /*Venda de Créditos*/
           SELECT 'S' AS TIPO,
                  NVL((SELECT MIN(T.CDTIPORECE)
                     FROM ITMENUCONFTE I, CAIXA C, TIPORECE T
                    WHERE (I.CDFILIAL = :CDFILIAL)
                      AND (I.IDTPBUTTON = '5')
                      AND (C.CDFILICONFTE = I.CDFILIAL)
                      AND (C.NRCONFTELA = I.NRCONFTELA)
                      AND (C.CDCAIXA = :CDCAIXA)
                      AND (C.NRORG = :NRORG)
                      AND (T.NRORG = C.NRORG)
                      AND (I.NRORG = C.NRORG)
                      AND (T.CDTIPORECE = I.CDIDENTBUTON)
                      AND (T.IDTIPORECE = '9')), '001') AS CDTIPORECE,
                  A.DTDEPCHEQUE, A.DTMOVIMCAIXA, A.NRENVELOPE, (NVL(A.VRMOVIVEND,0) - NVL(B.VRMOVIVEND,0)) AS VRMOVIVEND, A.VRMOVIVEOUT, A.VRCOTINDOUT
             FROM
              (SELECT M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE,
                      SUM(M.VRMOVIVEND) AS VRMOVIVEND, M.VRMOVIVEOUT, M.VRCOTINDOUT
                 FROM MOVCAIXA M, TIPORECE T
                WHERE (M.CDFILIAL = :CDFILIAL)
                  AND (M.CDCAIXA = :CDCAIXA)
                  AND (TO_CHAR(M.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX)
                  AND (M.IDTIPOMOVIVE = 'E')
                  AND (M.NRSEQVENDA IS NULL)
                  AND (M.DSMOVIVEND IS NULL)
                  AND (T.CDTIPORECE = M.CDTIPORECE)
                  AND (T.IDTIPORECE <> '7')
                  AND (M.NRORG = :NRORG)
                  AND (T.NRORG = M.NRORG)
               GROUP BY M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE, M.VRMOVIVEOUT, M.VRCOTINDOUT) A
               LEFT JOIN 
              (SELECT M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE,
                      SUM(M.VRMOVIVEND) AS VRMOVIVEND, M.VRMOVIVEOUT, M.VRCOTINDOUT
                 FROM MOVCAIXA M, TIPORECE T
                WHERE (M.CDFILIAL = :CDFILIAL)
                  AND (M.CDCAIXA = :CDCAIXA)
                  AND (TO_CHAR(M.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX)
                  AND (M.IDTIPOMOVIVE = 'S')
                  AND (M.NRSEQVENDA IS NULL)
                  AND (M.DSMOVIVEND IS NULL)
                  AND (T.CDTIPORECE = M.CDTIPORECE)
                  AND (T.IDTIPORECE <> '7')
                  AND (M.NRORG = :NRORG)
                  AND (T.NRORG = M.NRORG)
               GROUP BY M.CDTIPORECE, M.DTDEPCHEQUE, M.DTMOVIMCAIXA, M.NRENVELOPE, M.VRMOVIVEOUT, M.VRCOTINDOUT) B
            ON (A.CDTIPORECE = B.CDTIPORECE)
           /*Fim*/) A
        GROUP BY A.TIPO, A.CDTIPORECE, A.DTDEPCHEQUE, A.DTMOVIMCAIXA, A.NRENVELOPE, A.VRMOVIVEOUT, A.VRCOTINDOUT";
        
    const VERIFTURCAIXAAUX = "
    SELECT CDFILIAL, DTABERCAIORI, NRORG
        FROM TURCAIXA
        WHERE (CDFILIAL     = :CDFILIAL) AND
              (CDCAIXAORIG  = :CDCAIXA )  AND
              (NRORG        = :NRORG )  AND
              (TO_CHAR(DTABERCAIORI, 'DD/MM/YY HH24:MI:SS') = TO_CHAR(TO_DATE(:DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS'), 'DD/MM/YY HH24:MI:SS'))";
              
    const  INSERT_TURCAIXA = "
     INSERT INTO TURCAIXA
    (CDFILIAL, CDCAIXA, DTABERCAIX, CDOPERADOR, NRORG, DTFECHCAIX, DSOBSCAIXA,
    CDOPERABER, CDOPERFECH, DTMOVTURCAIX, CDCAIXAORIG, DTABERCAIORI)
    VALUES
    (:CDFILIAL, :CDCAIXACONT, TO_DATE(:DATAATUAL, 'DD/MM/YYYY HH24:MI:SS'), :CDOPERADOR, :NRORG,   TO_DATE(:DATAATUAL, 'DD/MM/YYYY HH24:MI:SS'),     NULL,
    :CDOPERADOR,      :CDOPERADOR,   TO_DATE(:DTMOVTURCAIX, 'DD/MM/YYYY HH24:MI:SS'), :CDCAIXA, TO_DATE(:DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS'))";
         
    const GET_NRSEQUMOVI = "
    SELECT MAX(NRSEQUMOVI) AS NRSEQUMOVI 
        FROM MOVCAIXA";
    
    const INSERT_MOVCAIXA = "
      INSERT INTO MOVCAIXA
        (CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI, IDTIPOMOVIVE, DSMOVIVEND,
         VRMOVIVEND, NRSEQVENDA, CDTIPORECE, DTHRINCMOV, DTDEPCHEQUE, NRCONTRAVALE,
         DTMOVIMCAIXA, NRENVELOPE, VRMOVIVEOUT, VRCOTINDOUT, IDIMPMOVCAIXA, NRORG)
        VALUES
        (:CDFILIAL, :CDCAIXACONT, TO_DATE(:DATAATUAL, 'DD/MM/YY HH24:MI:SS'), :NRSEQUMOVI, :IDTIPOMOVIVE, 'Transf. de movimentação do caixa',
         :VRMOVIVEND, NULL, :CDTIPORECE, TO_DATE(:DATAATUAL, 'DD/MM/YY HH24:MI:SS'), :DTDEPCHEQUE, 0,
         TO_DATE(:DTMOVIMCAIXA, 'DD/MM/YY HH24:MI:SS'), :NRENVELOPE, :VRMOVIVEOUT, :VRCOTINDOUT, 'S', :NRORG) ";
         
    const SELECT_SYSDATE = "
     SELECT TO_CHAR(SYSDATE, 'DD/MM/YYYY HH24:MI:SS') AS DATAATUAL FROM DUAL";
     
    const DELETE_MOVCAIXA = "
    DELETE FROM MOVCAIXA
        WHERE (CDFILIAL   = :CDFILIAL  ) AND
          (CDCAIXA    = :CDCAIXA   ) AND
          (TO_CHAR(DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIXCONT) AND 
          (NRORG      = :NRORG     )
    ";
    
    const DELETE_TURCAIXA = "
    DELETE FROM TURCAIXA
        WHERE (CDFILIAL   = :CDFILIAL  ) AND
              (CDCAIXA    = :CDCAIXA   ) AND
              (TO_CHAR(DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIXCONT) AND 
              (NRORG      = :NRORG     )
    ";
    
    const GET_DTABERCAIX_CONTROLE = "
    SELECT DTABERCAIX
        FROM TURCAIXA
        WHERE (CDFILIAL     = :CDFILIAL  ) AND
              (CDCAIXA      = :CDCAIXA   ) AND
              (CDCAIXAORIG  = :CDCAIXAORIG) AND
              (TO_CHAR(DTABERCAIORI, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX) AND
              (NRORG        = :NRORG)";
              
     const SALDO_CAIXA = "
        SELECT SC.CDFILIAL, SC.CDCAIXA, SC.DTMOVSALDCXA, SC.CDTIPORECE, SC.CDCLIENTECXA, SC.VRMOVDIACXA,
               SC.VRSALDACUCXA, SC.CDOPERADOR, SC.DTATUSALDCXA, TR.NMTIPORECE, CL.NRINSJURCLIE,
               SC.VRENTRADACXA, SC.VRSAIDACXA,
               CL.IDTPIJURCLIE, CL.NMRAZSOCCLIE, NVL(SAN.VRSALDANT, 0) AS VRSALDANT
        FROM SALDOCAIXA SC
           JOIN TIPORECE TR ON 
              (SC.CDTIPORECE   = TR.CDTIPORECE)     AND
              (SC.NRORG        = TR.NRORG)  
           LEFT JOIN CLIENTE CL ON
              (SC.CDCLIENTECXA = CL.CDCLIENTE)   AND
              (SC.NRORG        = CL.NRORG)   
           LEFT JOIN (SELECT SA.CDFILIAL, SA.CDCAIXA, SA.CDTIPORECE, SA.CDCLIENTECXA, MAX(SA.VRSALDACUCXA) AS VRSALDANT,SA.NRORG
              FROM SALDOCAIXA SA
              WHERE (SA.CDFILIAL     = :P_CDFILIAL)     AND
                    (SA.CDCAIXA      = :P_CDCAIXA)      AND
                    (SA.DTMOVSALDCXA = (SELECT MAX(SD.DTMOVSALDCXA)
                                        FROM SALDOCAIXA SD
                                        WHERE (SD.CDFILIAL     = SA.CDFILIAL)     AND
                                              (SD.CDCAIXA      = SA.CDCAIXA)      AND
                                              (SD.CDTIPORECE   = SA.CDTIPORECE)   AND
                                              (SD.CDCLIENTECXA = SA.CDCLIENTECXA) AND
                                              (DTMOVSALDCXA < :P_DTMOVSALDCXA))) AND
                    (SA.NRORG = :P_NRORG)
             GROUP BY SA.CDFILIAL, SA.CDCAIXA, SA.CDTIPORECE, SA.CDCLIENTECXA,SA.NRORG) SAN ON
              (SC.CDFILIAL     = SAN.CDFILIAL)   AND
              (SC.CDCAIXA      = SAN.CDCAIXA)    AND
              (SC.CDTIPORECE   = SAN.CDTIPORECE) AND
              (SC.NRORG        = SAN.NRORG)      AND
              (SC.CDCLIENTECXA = SAN.CDCLIENTECXA)
        WHERE (SC.CDFILIAL     = :P_CDFILIAL)       AND
              (SC.CDCAIXA      = :P_CDCAIXA)        AND
              (SC.DTMOVSALDCXA = :P_DTMOVSALDCXA)   AND
              (SC.NRORG        = :P_NRORG)            
        ORDER BY SC.CDTIPORECE, SC.CDCLIENTECXA";
        
    const GET_VENDAS_IRREGULARES = "
    SELECT X.CDFILIAL, X.CDCAIXA, X.NRSEQVENDA, X.DTVENDA, TO_CHAR(VRITEMVENDA, 'FM999G999G999D90', 'nls_numeric_characters='',.''') VRITEMVENDA, 
           TO_CHAR(VRMOVCAIXA, 'FM999G999G999D90', 'nls_numeric_characters='',.''') VRMOVCAIXA, TO_CHAR((VRMOVCAIXA-VRITEMVENDA), 'FM999G999G999D90', 'nls_numeric_characters='',.''') AS DIF
      FROM (SELECT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.DTVENDA, V.IDTPEMISVEND, SUM(TRUNC(I.QTPRODVEND*(I.VRUNITVEND+NVL(I.VRUNITVENDCL,0)),2) -NVL(I.VRDESITVEND,0) +NVL(I.VRACRITVEND,0)) AS VRITEMVENDA
              FROM ITEMVENDA I, VENDA V
             WHERE V.IDTPEMISVEND <> 'E'
               AND V.IDSITUVENDA   = 'O'
               AND I.CDFILIAL      = V.CDFILIAL
               AND I.CDCAIXA       = V.CDCAIXA
               AND I.NRSEQVENDA    = V.NRSEQVENDA  
               AND V.CDFILIAL      = :CDFILIAL
               AND V.CDCAIXA       = :CDCAIXA
               AND V.NRORG         = :NRORG
               AND TO_CHAR(V.DTABERTUR, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX
            GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.DTVENDA, V.IDTPEMISVEND )X,
           (SELECT M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA,  
                   SUM(CASE IDTIPOMOVIVE WHEN 'E' THEN +VRMOVIVEND WHEN 'T' THEN +VRMOVIVEND WHEN 'S' THEN -VRMOVIVEND ELSE 0 END) AS VRMOVCAIXA
              FROM MOVCAIXA M, VENDA V
             WHERE V.IDTPEMISVEND <> 'E'
               AND IDSITUVENDA     = 'O'
               AND M.CDFILIAL      = V.CDFILIAL
               AND M.CDCAIXA       = V.CDCAIXA
               AND M.NRSEQVENDA    = V.NRSEQVENDA 
               AND M.IDTIPOMOVIVE  <> 'C'
               AND V.CDFILIAL      = :CDFILIAL
               AND V.CDCAIXA       = :CDCAIXA
               AND V.NRORG         = :NRORG
               AND TO_CHAR(V.DTABERTUR, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX
            GROUP BY M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA) Y                  
     WHERE X.CDFILIAL   = Y.CDFILIAL
       AND X.CDCAIXA    = Y.CDCAIXA
       AND X.NRSEQVENDA = Y.NRSEQVENDA
       AND VRITEMVENDA <> VRMOVCAIXA";    
       
    const ACMCANCELAMOV = "
    UPDATE MOVCAIXA
       SET IDTIPOMOVIVE = 'C',
           DSMOVIVEND   = :DSMOVIVEND 
     WHERE (CDFILIAL   = :CDFILIAL  ) AND
           (CDCAIXA    = :CDCAIXA   ) AND
           (DTABERCAIX = :DTABERCAIX) AND
           (NRSEQUMOVI = :NRSEQUMOVI)";
           
    const NRSEQVENDA_VALIDO = "
        SELECT DISTINCT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, FLC.NMFILIAL
          FROM VENDA V, (SELECT DISTINCT CDFILIAL, NMFILIAL, CDLOJA, CDCAIXA FROM V_FILIALLOJACAIXA WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA) FLC
         WHERE V.CDCAIXA   = FLC.CDCAIXA
           AND V.CDLOJA      = FLC.CDLOJA
           AND V.CDFILIAL    = FLC.CDFILIAL
           AND V.NRSEQVENDA = :NRSEQVENDA
           AND TO_CHAR(V.DTABERTUR, 'DD/MM/YYYY HH24:MI:SS') = :DTABERCAIX
    ";
    
    
    
    
    /**
     * ALTERAÇÕES PARA GERAÇÃO DE FINANCEIRO
     */ 
    
    // Constantes de Nota Fiscal de Entrada
    const IDBLOQUEADO  = 'S';
    const IDDOCFISCAL  = 'N';
    const IDDOCFINANC  = 'S';
    const IDENTRSAIDOP = 'E';
    const IDDOCCONTAB  = 'S';
    const IDSTATUS     = 'A'; //VERIFICAR SE COLOCA COMO PARAMETRO, VERIFICAR IDMOVFLUXO
    const APROVACAO    = 'A';
    const CANFIN       = 'S';
    const IDSISTEMA    = 'PAG';
    
    // Constantes de Conta a Pagar
    const NRSEQVENPAG  = '01';
    const NRPARCELAPAG = '00';
    const IDSTATPAG    = 'A';
    const QTDIASVENPAG = 0;
    const IDMOVFLUXO   = 'E'; //SE MUDAR O IDSTATUS TEM QUE RESPEITAR IDSTATUS == V ? 'P' : 'E'
    
    
    /**
     * Gera Nota Fiscal Pagamento
     */
    public function geraNFPagamento(
        $nrorg, $cdfilial, $cdfilirece, $cdserieentr, $cdforneced,
        $cdtipooper, $vrnotafisc, $dtatuavenpag, $cdtpctpagar
    ){
        
        // Variáveis de inicialização - Nota Fiscal de Entrada
        $nrlanctonf     = $this->newCode->getCodeByTableName('LANCNF/FILIAL'.$cdfilial, $nrorg);
        
        $paramfilial    = $this->buscaParamfilial($nrorg, $cdfilial);
        $nrtamnumnfut   = $paramfilial['NRTAMNUMNFUT'] > 0 ? $paramfilial['NRTAMNUMNFUT'] : 9;
        $nrnotafisc     = $this->newCode->getCodeByTableName('NRNOTAFISCPAG', $nrorg, 1, $nrtamnumnfut);
        
        $filial         = $this->buscaFilial($nrorg, $cdfilial);
        $cdempresa      = $filial['CDEMPRESA'];
        $cdinscesta     = $filial['CDINSCESTA'];
        $cdinscmuni     = $filial['CDINSCMUNI'];

        $fornecedor     = $this->buscaFornecedor($nrorg, $cdforneced);
        $idtpijurnf     = $fornecedor['IDTPIJURFORN'];
        $nrinsjurnf     = $fornecedor['NRINSJURFORN'];
        
        $dtemissao      = $dtatuavenpag;
        $dtentrsaid     = $dtatuavenpag;
        $dtliberacaofin = DateUtil::getDataAtual();//necessario ou nao
        $dtlancamennf   = DateUtil::getDataAtual();
        $dtinclusao     = DateUtil::getDataAtual();
        
        $cdoperactpg    = $this->environment->getCdOperador();
        $vrprodutos     = $vrnotafisc;
        
        // Variáveis de inicialização - Conta a Pagar
        $cdfimovflpag = $cdfilial;
        $dtorigvenpag = $dtatuavenpag;
        $dtmovfluxo   = $dtatuavenpag;
        
        $tipoctpagar  = $this->buscaTipoctpagar($nrorg, $cdtpctpagar);
        $cdclassfina  = $tipoctpagar['CDCLASSFINA'];
        $dscontapag   = $tipoctpagar['DSDEFACTPAG'];
        
        $vrorigpag    = $vrnotafisc;
        $vratuapag    = $vrnotafisc;
        
        // Gera Nota Fiscal Pagar
       $params_notafiscal = array(
            ':NRORG'          => $nrorg,
            ':CDFILIAL'       => $cdfilial,
            ':NRLANCTONF'     => $nrlanctonf,
            ':CDEMPRESA'      => $cdempresa,
            ':CDINSCESTA'     => $cdinscesta,
            ':CDINSCMUNI'     => $cdinscmuni,
            ':CDFILIRECE'     => $cdfilirece,
            ':IDTPIJURNF'     => $idtpijurnf,
            ':NRINSJURNF'     => $nrinsjurnf,
            ':CDFORNECED'     => $cdforneced,
            ':IDBLOQUEADO'    => self::IDBLOQUEADO,
            ':IDDOCFISCAL'    => self::IDDOCFISCAL,
            ':IDDOCFINANC'    => self::IDDOCFINANC,
            ':VRPRODUTOS'     => $vrprodutos,
            ':DTLIBERACAOFIN' => $dtliberacaofin,
            ':VRNOTAFISC'     => $vrnotafisc,
            ':NRNOTAFISC'     => $nrnotafisc,
            ':DTEMISSAO'      => $dtemissao,
            ':DTENTRSAID'     => $dtentrsaid,
            ':IDENTRSAIDOP'   => SELF::IDENTRSAIDOP,
            ':CDTIPOOPER'     => $cdtipooper,
            ':CDSERIEENTR'    => $cdserieentr,
            ':IDDOCCONTAB'    => SELF::IDDOCCONTAB,
            ':IDSTATUS'       => self::IDSTATUS,
            ':IDSISTEMA'      => SELF::IDSISTEMA,
            ':DTLANCAMENNF'   => $dtlancamennf,
            ':CDOPERACTPG'    => $cdoperactpg,
            ':DTINCLUSAO'     => $dtinclusao
        );
        $this->conn->executeQuery(self::INSERT_NOTAFISCAL , $params_notafiscal);
        $this->entityManager->flush();
        
        // Gera Conta Pagar
        $params_contapagar = array(
            ':NRORG'        => $nrorg,
            ':CDFILIAL'     => $cdfilial,
            ':NRLANCTONF'   => $nrlanctonf,
            ':NRSEQVENPAG'  => self::NRSEQVENPAG,
            ':NRPARCELAPAG' => self::NRPARCELAPAG,
            ':DTORIGVENPAG' => $dtorigvenpag,
            ':VRATUAPAG'    => $vratuapag,
            ':CDTPCTPAGAR'  => $cdtpctpagar,
            ':DTATUAVENPAG' => $dtatuavenpag,
            ':VRORIGPAG'    => $vrorigpag,
            ':IDSTATPAG'    => self::IDSTATPAG,
            ':QTDIASVENPAG' => self::QTDIASVENPAG,
            ':DSCONTAPAG'   => $dscontapag,
            ':CDCLASSFINA'  => $cdclassfina,
            ':CDEMPRESA'    => $cdempresa,
            ':DTMOVFLUXO'   => $dtmovfluxo,
            ':IDMOVFLUXO'   => self::IDMOVFLUXO,
            ':CDFIMOVFLPAG' => $cdfimovflpag,
            ':DSCONTAPAG'   => $dscontapag,
            ':DTINCLUSAO'   => $dtinclusao
        );
        $this->conn->executeQuery(self::INSERT_CONTAPAGAR , $params_contapagar);
        $this->entityManager->flush();
        
        return $nrlanctonf;
    }
    
    /**
     * Salva Documento Visculado Sangria
     */
    public function saveDocVinculado(
        $nrorg, $cdfilial, $cdcaixa, $dtabercaix, $nrsequmovi, $idtpdocsang, $cdfilialnf, $nrlanctonf, $cdtpbaixapag, $cdreducont
    ){
        $params_docsangria = array(
            ':NRORG'        => $nrorg,
            ':CDFILIAL'     => $cdfilial,
            ':CDCAIXA'      => $cdcaixa,
            ':DTABERCAIX'   => $dtabercaix,
            ':NRSEQUMOVI'   => $nrsequmovi,
            ':IDTPDOCSANG'  => $idtpdocsang,
            ':CDFILIALNF'   => $cdfilialnf,
            ':NRLANCTONF'   => $nrlanctonf,
            ':CDTPBAIXAPAG' => $cdtpbaixapag,
            ':CDREDUCONT'   => $cdreducont
        );
        $this->conn->executeQuery(self::INSERT_DOCSANGRIA , $params_docsangria);
        $this->entityManager->flush();
    }
    
    /**
     * Deleta Documento Visculado Sangria
     */
    public function deleteDocVinculado(
        $nrorg, $cdfilial, $cdcaixa, $dtabercaix, $nrsequmovi, $idtpdocsang, $cdfilialnf, $nrlanctonf
    ){
        if($idtpdocsang == 'R'){
            $params_notafiscal_contapagar = array(
                ':NRORG'        => $nrorg,
                ':CDFILIAL'     => $cdfilialnf,
                ':NRLANCTONF'   => $nrlanctonf,
            );
            $this->conn->executeQuery(self::DELETE_NOTAFISCAL , $params_notafiscal_contapagar);
            $this->conn->executeQuery(self::DELETE_CONTAPAGAR , $params_notafiscal_contapagar);
            $this->entityManager->flush();
        }
        
        $params_docsangria = array(
            ':NRORG'       => $nrorg,
            ':CDFILIAL'    => $cdfilial,
            ':CDCAIXA'     => $cdcaixa,
            ':DTABERCAIX'  => $dtabercaix,
            ':NRSEQUMOVI'  => $nrsequmovi,
            ':IDTPDOCSANG' => $idtpdocsang,
            ':CDFILIALNF'  => $cdfilialnf,
            ':NRLANCTONF'  => $nrlanctonf
        );
        $this->conn->executeQuery(self::DELETE_DOCSANGRIA , $params_docsangria);
        $this->entityManager->flush();
    }
    
    /**
     * Gera Financeiro
     */
    public function generateFinanceiro(
        $caixa, $lancamento, $totalizador, $documentos
    ){
        try{
            foreach($documentos as $documento){
                $cdfilial   = $documento['CDFILIALNF'];
                $nrlanctonf = $documento['NRLANCTONF'];
                $cdfiliproc = $this->environment->getCdFilial();
                
                $procedure = new StoredProcedure($this->conn, 'GERA_DADOS_FINANCEIRO');
                $procedure->addParam(new Param('P_CDFILIAL'  , Param::PARAM_INPUT, $cdfilial       , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf     , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $cdfiliproc     , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_IDOPERACAO', Param::PARAM_INPUT, SELF::APROVACAO , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_RESULTADO' , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
                $resultado = $procedure->execute();
        
                if(count($resultado) > 0)
                    throw new \Exception("Operação bloqueada. Ocorreu um erro na geração do financeiro.");
            }
            
            if($totalizador['VRDIFMOVIVEND'] > 0){
                // Variáveis de inicialização - Movcaixa
                $params_nrsequmovi = array(
                    ':NRORG'      => $caixa['NRORG'],
                    ':CDFILIAL'   => $caixa['CDFILIAL'],
                    ':CDCAIXA'    => $caixa['CDCAIXA'],
                    ':DTABERCAIX' => $caixa['DTABERCAIX']
                );
                $movcaixa   = $this->conn->fetchAssoc(self::SELECT_MAX_NRSEQUMOVI , $params_nrsequmovi);
                $sequencial = $movcaixa['NRSEQUMOVI'] + 1;
                $nrsequmovi = str_pad($sequencial, 10, "0", STR_PAD_LEFT);
                
                /**
                 * ATENÇÃO
                 * 
                 * Caso altere a descrição, deve ser alterado tambem o delete 
                 * da rotina de cancelFinanceiro para que ela consiga localizar
                 * o lançamento de suprimento gerado caso aja devolução
                 * 
                 */ 
                $dsmovivend = 'Devolução gerada através do movimento de sangria '.$lancamento['NRSEQUMOVI'].'.';
                
                $params_movcaixa = array(
                    ':NRORG'        => $caixa['NRORG'],
                    ':CDFILIAL'     => $caixa['CDFILIAL'],
                    ':CDCAIXA'      => $caixa['CDCAIXA'],   
                    ':DTABERCAIX'   => $caixa['DTABERCAIX'],
                    ':NRSEQUMOVI'   => $nrsequmovi,
                    ':DTHRINCMOV'   => DateUtil::getDataAtual(),
                    ':DTMOVIMCAIXA' => DateUtil::getDataAtual(),
                    ':IDTIPOMOVIVE' => 'U',
                    ':CDTIPORECE'   => $lancamento['CDTIPORECE'],
                    ':DSMOVIVEND'   => $dsmovivend,
                    ':VRMOVIVEND'   => $totalizador['VRDIFMOVIVEND']
                );
                $this->conn->executeQuery(self::INSERT_MOVCAIXA_SUPRIMENTO , $params_movcaixa);
            }

        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Cancela Financeiro
     */
    public function cancelFinanceiro(
        $caixa, $lancamento, $totalizador, $documentos
    ){
        try{
            foreach($documentos as $documento){
                $cdfilial   = $documento['CDFILIALNF'];
                $nrlanctonf = $documento['NRLANCTONF'];
                $cdfiliproc = $this->environment->getCdFilial();
                
                $procedure = new StoredProcedure($this->conn, 'CANCELA_DADOS_FINANCEIRO');
                $procedure->addParam(new Param('P_CDFILIAL'  , Param::PARAM_INPUT, $cdfilial       , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf     , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_IDCANFIN'  , Param::PARAM_INPUT, SELF::CANFIN    , Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_RETORNO'   , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
                $resultado = $procedure->execute();
                
                if(count($resultado) > 0)
                    throw new \Exception("Operação bloqueada. Ocorreu um erro nO cancelamento do financeiro.");
                    
                $params_contapagar = array(
                    ':NRORG'       => $caixa['NRORG'],
                    ':CDFILIAL'    => $documento['CDFILIALNF'],
                    ':NRLANCTONF'  => $documento['NRLANCTONF'],
                    ':CDTPCTPAGAR' => $documento['CDTPCTPAGAR']
                );
                $this->conn->executeQuery(self::UPDATE_CONTAPAGAR, $params_contapagar);
            }
            
            $params_movcaixa = array(
                ':NRORG'      => $caixa['NRORG'],
                ':CDFILIAL'   => $caixa['CDFILIAL'],
                ':CDCAIXA'    => $caixa['CDCAIXA'],
                ':DTABERCAIX' => $caixa['DTABERCAIX'],
                ':DSMOVIVEND' => 'Devolução gerada através do movimento de sangria '.$lancamento['NRSEQUMOVI'].'.'
            );
            
            $this->conn->executeQuery(self::DELETE_MOVCAIXA_SUPRIMENTO, $params_movcaixa);
            $this->entityManager->flush();
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Passar para o FACTORY
     */
     
    /**
     * Busca Parâmetros da Filial
     * @param $nrorg
     * @param $cdfilial
     * @return Tipoctpagar
     */
    public function buscaParamfilial($nrorg, $cdfilial){
        $params = array(
            ':NRORG'    => $nrorg,
            ':CDFILIAL' => $cdfilial
        );
        $result = $this->conn->fetchAll(self::SELECT_PARAMFILIAL, $params);
        return $result[0];
    }
    
    const SELECT_PARAMFILIAL = "
        SELECT NRTAMNUMNFUT FROM PARAMFILIAL
    ";
    
    /**
     * Busca Filial
     * @param $nrorg
     * @param $cdfilial
     * @return Filial
     */
     public function buscaFilial($nrorg, $cdfilial){
        $params = array(
            ':NRORG'     => $nrorg,
            ':CDFILIAL'  => $cdfilial
        );
        $result = $this->conn->fetchAll(self::SELECT_FILIAL, $params);
        return $result[0];
    }
    
    const SELECT_FILIAL = "
        SELECT CDEMPRESA, CDINSCESTA, CDINSCMUNI
          FROM FILIAL
         WHERE NRORG    = :NRORG
           AND CDFILIAL = :CDFILIAL
    ";
    
    /**
     * Busca Fornecedor
     * @param $nrorg
     * @param $cdforneced
     * @return Fornecedor
     */
    public function buscaFornecedor($nrorg, $cdforneced){
        $params = array(
            ':NRORG'      => $nrorg,
            ':CDFORNECED' => $cdforneced
        );
        $result = $this->conn->fetchAll(self::SELECT_FORNECEDOR, $params);
        return $result[0];
    }
    
    const SELECT_FORNECEDOR = "
        SELECT IDTPIJURFORN, NRINSJURFORN
          FROM FORNECEDOR
         WHERE NRORG      = :NRORG
           AND CDFORNECED = :CDFORNECED
    ";
    
    /**
     * Busca Tipoctpagar
     * @param $nrorg
     * @param $cdtpctpagar
     * @return Tipoctpagar
     */
    public function buscaTipoctpagar($nrorg, $cdtpctpagar){
        $params = array(
            ':NRORG'      => $nrorg,
            ':CDTPCTPAGAR' => $cdtpctpagar
        );
        $result = $this->conn->fetchAll(self::SELECT_TIPOCTPAGAR, $params);
        return $result[0];
    }
    
    const SELECT_TIPOCTPAGAR = "
        SELECT CDCLASSFINA, DSDEFACTPAG
          FROM TIPOCTPAGAR
         WHERE NRORG       = :NRORG
           AND CDTPCTPAGAR = :CDTPCTPAGAR
    ";
    
    const SELECT_MAX_NRSEQUMOVI = "
    SELECT MAX(NRSEQUMOVI) AS NRSEQUMOVI 
      FROM MOVCAIXA
     WHERE (NRORG       = :NRORG      )
       AND (CDFILIAL    = :CDFILIAL   )
       AND (CDCAIXA     = :CDCAIXA    )
       AND (DTABERCAIX  = :DTABERCAIX )";

    const INSERT_NOTAFISCAL = "
      INSERT INTO NOTAFISCAL(
            NRORG, CDFILIAL, NRLANCTONF, CDEMPRESA, CDINSCESTA, CDINSCMUNI, CDFILIRECE, IDTPIJURNF, NRINSJURNF, CDFORNECED,
            IDBLOQUEADO, IDDOCFISCAL, IDDOCFINANC, VRPRODUTOS, DTLIBERACAOFIN, VRNOTAFISC, NRNOTAFISC, DTEMISSAO, DTENTRSAID,
            IDENTRSAIDOP, CDTIPOOPER, CDSERIEENTR, IDDOCCONTAB, IDSTATUS, IDSISTEMA, DTLANCAMENNF, CDOPERACTPG, DTINCLUSAO
        )VALUES(
            :NRORG, :CDFILIAL, :NRLANCTONF, :CDEMPRESA, :CDINSCESTA, :CDINSCMUNI, :CDFILIRECE, :IDTPIJURNF, :NRINSJURNF, :CDFORNECED,
            :IDBLOQUEADO, :IDDOCFISCAL, :IDDOCFINANC, :VRPRODUTOS, SYSDATE, :VRNOTAFISC, :NRNOTAFISC, TRUNC(TO_DATE(:DTEMISSAO, 'DD/MM/YY HH24:MI:SS')), TRUNC(TO_DATE(:DTENTRSAID, 'DD/MM/YY HH24:MI:SS')),
            :IDENTRSAIDOP, :CDTIPOOPER, :CDSERIEENTR, :IDDOCCONTAB, :IDSTATUS, :IDSISTEMA, SYSDATE, :CDOPERACTPG, SYSDATE
        )";
         
    const INSERT_CONTAPAGAR = "
        INSERT INTO CONTAPAGAR(
            NRORG, CDFILIAL, NRLANCTONF, NRSEQVENPAG, NRPARCELAPAG, DTORIGVENPAG, VRATUAPAG, 
            CDTPCTPAGAR, DTATUAVENPAG, VRORIGPAG, IDSTATPAG, QTDIASVENPAG, DSCONTAPAG, CDCLASSFINA,
            CDEMPRESA, DTINCLUSAO
        )VALUES(
            :NRORG, :CDFILIAL, :NRLANCTONF, :NRSEQVENPAG, :NRPARCELAPAG, :DTORIGVENPAG, :VRATUAPAG,
            :CDTPCTPAGAR, TRUNC(TO_DATE(:DTATUAVENPAG, 'DD/MM/YY HH24:MI:SS')), :VRORIGPAG, :IDSTATPAG, :QTDIASVENPAG, :DSCONTAPAG, :CDCLASSFINA,
            :CDEMPRESA, SYSDATE
        )";
        
    const INSERT_DOCSANGRIA = "
        INSERT INTO DOCSANGRIA(
            NRORG, CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI, IDTPDOCSANG, CDFILIALNF, NRLANCTONF, CDTPBAIXAPAG, CDREDUCONT
        )VALUES(
            :NRORG, :CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQUMOVI, :IDTPDOCSANG, :CDFILIALNF, :NRLANCTONF, :CDTPBAIXAPAG, :CDREDUCONT
        )";
        
    const INSERT_MOVCAIXA_SUPRIMENTO = "
        INSERT INTO MOVCAIXA(
            NRORG, CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI, IDTIPOMOVIVE, CDTIPORECE, DTHRINCMOV, DTMOVIMCAIXA, DSMOVIVEND, VRMOVIVEND
        )VALUES(
            :NRORG, :CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQUMOVI, :IDTIPOMOVIVE, :CDTIPORECE, SYSDATE, TRUNC(SYSDATE), :DSMOVIVEND, :VRMOVIVEND
        )";
        
    const UPDATE_CONTAPAGAR = "
        UPDATE CONTAPAGAR
           SET CDTPCTPAGAR = :CDTPCTPAGAR
         WHERE CDFILIAL    = :CDFILIAL
           AND NRLANCTONF  = :NRLANCTONF
    ";
        
    const DELETE_NOTAFISCAL = "
    DELETE FROM NOTAFISCAL
        WHERE (NRORG      = :NRORG     )
          AND (CDFILIAL   = :CDFILIAL  )
          AND (NRLANCTONF = :NRLANCTONF)
    ";
    
    const DELETE_CONTAPAGAR = "
    DELETE FROM CONTAPAGAR
        WHERE (NRORG      = :NRORG     )
          AND (CDFILIAL   = :CDFILIAL  )
          AND (NRLANCTONF = :NRLANCTONF)
    ";
    
    const DELETE_DOCSANGRIA = "
    DELETE FROM DOCSANGRIA
        WHERE (NRORG       = :NRORG      )
          AND (CDFILIAL    = :CDFILIAL   )
          AND (CDCAIXA     = :CDCAIXA    )
          AND (DTABERCAIX  = :DTABERCAIX )
          AND (NRSEQUMOVI  = :NRSEQUMOVI )
          AND (IDTPDOCSANG = :IDTPDOCSANG)
          AND (CDFILIALNF  = :CDFILIALNF )
          AND (NRLANCTONF  = :NRLANCTONF )
    ";
    
    const DELETE_MOVCAIXA_SUPRIMENTO = "
        DELETE FROM MOVCAIXA
         WHERE (NRORG       = :NRORG      )
           AND (CDFILIAL    = :CDFILIAL   )
           AND (CDCAIXA     = :CDCAIXA    )
           AND (DTABERCAIX  = :DTABERCAIX )
           AND (DSMOVIVEND  = :DSMOVIVEND )";
    
    /**
     * Formata Data
     * @param $data
     * @return \DateTime|string
     */
    public static function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    
    public function replicaMovcaixaToCNC ($data){
        
        $data = str_replace(' 00:00:00', '', $data);
        
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'REPLICACAO_FOS_CNC');
        $storedProcedure->addParam($this->buildInputParam('P_DATAINI', $data));
        $storedProcedure->addParam($this->buildInputParam('P_DATAFIN', $data));
        
        $result = $storedProcedure->execute();;
        
        return $result;
        
    }

    private function buildInputParam ($paramName, $paramValue) {
        return new Param($paramName, Param::PARAM_INPUT, $paramValue, Param::PARAM_TYPE_STR, strlen($paramValue));
    }
    
    public function deletaFinanceiroCNC ($row){
        
        $params = array(
            ':CDFILIAL' => $row['CDFILIAL'],
            ':NRLANCTONF' => $row['NRLANCTONFCNC']
        );
        $result = $this->conn->fetchAssoc(self::GET_NOTA_BOLETO_CONTA_INFO, $params);
        
        $paramsDesvincula = array(
            ':CDFILIAL' => $row['CDFILIAL'],
            ':NRCONSMOVCAIXA' => $row['NRCONSMOVCAIXA']
        );
        
        $paramsNota = array(
            ':NRLANBOLREC' =>  $result['NRLANBOLREC'],
            ':NRLANCTONF' =>  $result['NRLANCTONF'],
            ':CDFILIAL' =>  $result['CDFILIAL']
        );
        $verifyBoletConta = $this->conn->fetchAssoc(self::VERIFY_BOLET_CONTA, $paramsNota);
        
        if (($verifyBoletConta['TOT'] == '1') || ($verifyBoletConta['TOT'] == NULL) || ($verifyBoletConta == false) || ((float)$verifyBoletConta['TOT'] == 1)) {
            
            //Remove o boleto da conta
            $this->conn->executeQuery(self::UPDATA_CONTARECEBER_CNC, $paramsNota);
            // desassocia o boleto da conta para poder deletar o mesmo
            // deleta tipo de recebimento do boleto
            $this->conn->executeQuery(self::DELETE_TPRECBOLETO_CNC, $paramsNota);
            // deleta boleto pois so existe uma conta associada
            $this->conn->executeQuery(self::DELETE_BOLETRECEBER_CNC, $paramsNota);
            // deleta ajuste da conta a receber
            $this->conn->executeQuery(self::DELETE_AJCTRECEBER_CNC, $paramsNota);
            // deleta o conta a receber
            $this->conn->executeQuery(self::DELETE_CONTARECEBER_CNC, $paramsNota);
            
            $this->conn->executeQuery(self::DESVINCULA_FINANCEIRO_CONSMOVCAIXA, $paramsDesvincula);
            // deleta nota fiscal
            $this->conn->executeQuery(self::DELETE_NOTAFISCAL_CNC, $paramsNota);
        }else if (((float)$verifyBoletConta['TOT'] > 1)){
            //Remove o boleto da conta
            $this->conn->executeQuery(self::UPDATA_CONTARECEBER_CNC, $paramsNota);
            
            $verifyBoletConta = $this->conn->fetchAssoc(self::VERIFY_BOLET_CONTA, $paramsNota);
            $paramsBolet = array(
                ':NRLANBOLREC' =>  $verifyBoletConta['NRLANBOLREC'],
                ':CDFILIAL' =>  $verifyBoletConta['CDFILIAL'],
                ':TOTBOLET' => (float)$verifyBoletConta['TOTBOLET']
            );
            // Primeiro corrigimos o boleto depois deletamos conta e nota fiscal para serem gerados novamente apos a edicao.
            // remove valor da conta do tipo de recebimento do boleto 
            $this->conn->executeQuery(self::UPDATE_TPRECBOLETO_CNC, $paramsBolet);
            // remove o valor da conta do boleto
            $this->conn->executeQuery(self::UPDATE_BOLETRECEBER_CNC, $paramsBolet);
            // deleta ajuste da conta a receber
            $this->conn->executeQuery(self::DELETE_AJCTRECEBER_CNC, $paramsNota);
            // deleta o conta a receber
            $this->conn->executeQuery(self::DELETE_CONTARECEBER_CNC, $paramsNota);
            
            $this->conn->executeQuery(self::DESVINCULA_FINANCEIRO_CONSMOVCAIXA, $paramsDesvincula);
            // deleta nota fiscal
            $this->conn->executeQuery(self::DELETE_NOTAFISCAL_CNC, $paramsNota);
        }
        
        return true;
    }
    
    public function deleteItemcorvenda ($row){
        $params = array(
            ':NRCONSMOVCAIXA' => $row['NRCONSMOVCAIXA']
        );
        $result = $this->conn->fetchAll(self::GET_ITEMCORVENDA_TO_CONSMOVCAIXA, $params);
        
        foreach ($result as $value) {
            $deleteParams = array(
                ':NRCONSMOVCAIXA' => $value['NRCONSMOVCAIXA'],
                ':NRITEMCORVENDA' => $value['NRITEMCORVENDA']
            );
            $this->conn->executeQuery(self::DELETE_ITEMCORVENDA, $deleteParams);    
        }
        
        return true;
    }
    
    public function deleteDivconciliacao ($row){
        $paramsDiv = array(
            ':NRCONSMOVCAIXA' => $row['NRCONSMOVCAIXA']
        );
        $this->conn->executeQuery(self::DELETE_DIVCONCILIACAO, $paramsDiv);
        return true;
    }
    
    public function desconcilia ($row){
        $params = array(
            ':NRCONCLANC' => $row['NRCONCLANC'],
            ':CDFILIAL' => $row['CDFILIAL'],
            ':NRORG' => $row['NRORG']
        );
        
        $this->conn->executeQuery(self::DESCONCILIA_RETFINANCADQ, $params);
        $this->conn->executeQuery(self::DESCONCILIA_RETVENDAADQ, $params);
        $this->conn->executeQuery(self::DESCONCILIA_CONSMOVCAIXA, $params);
        $this->conn->executeQuery(self::DELETE_CONCILIACAOLANC, $params);
        
        return true;
    }
    
    
    public function deletaConsmovcaixa ($row){
        $params = array(
            ':NRCONSMOVCAIXA' => $row['NRCONSMOVCAIXA']
        );
        
        $this->conn->executeQuery(self::DELETE_CONSMOVCAIXA, $params);
        
        return true;
    }
    
    // Delete Consmovcaixa
    
    const DELETE_CONSMOVCAIXA = "
   		 DELETE FROM CONSMOVCAIXA
		 WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    // Desconcilia
    
    const DESCONCILIA_RETFINANCADQ = "
   		 UPDATE RETFINANCADQ
		  SET NRCONCLANC = null,
		      IDCONCILIADO = 'P',
		      IDSITUACAO = 'N'
		 WHERE NRORG = :NRORG
           AND NRCONCLANC = :NRCONCLANC
    ";

    const DESCONCILIA_RETVENDAADQ = "
		 UPDATE RETVENDAADQ
		    SET NRCONCLANC = null,
		    	NRCONCLANCPGTO = null,
		    	IDCONCILIADO = 'P',
		    	IDSITUACAO = 'N',
                NRLANCTONF = null,
                CDFILIAL = null
		 WHERE NRCONCLANC = :NRCONCLANC
	";

    const DESVINCULA_FINANCEIRO_CONSMOVCAIXA = "
    	UPDATE CONSMOVCAIXA
           SET CDFILIALNF = NULL,
               NRLANCTONF= NULL
         WHERE CDFILIALNF = :CDFILIAL 
           AND NRCONSMOVCAIXA = :NRCONSMOVCAIXA 
	";

    const DESCONCILIA_CONSMOVCAIXA = "
    	UPDATE CONSMOVCAIXA
           SET NRCONCLANC = null,
               IDCONCILIADO = 'P',
               IDSITUACAO = 'N'
         WHERE NRCONCLANC = :NRCONCLANC
           AND CDFILIALNF = :CDFILIAL 
	";
	
	const DELETE_CONCILIACAOLANC = "
        DELETE FROM CONCILIACAOLANC
        WHERE NRCONCLANC = :NRCONCLANC
	";
    
    //Updates itemcorvenda
    // Tipo rece
    const UPDATE_TIPORECE = "
        UPDATE CONSMOVCAIXA SET CDTIPORECE = :TIPORECEANTIGO WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    //Data da venda
    const UPDATE_DATA_VENDA = "
        UPDATE CONSMOVCAIXA SET DTVENDA = :VRANTIGO WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    //NSU
    const UPDATE_NSU = "
        UPDATE CONSMOVCAIXA SET CDNSU = :VRANTIGO WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    //Valor
    const UPDATE_VALOR = "
        UPDATE CONSMOVCAIXA SET VRVEND = :VRANTIGO WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    //Data de vencimento
    const UPDATE_DATA_VENCIMENTO = "
        UPDATE CONSMOVCAIXA SET DTVENCIMENTO = :VRANTIGO WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    //AUTORIZACAO
    const UPDATE_AUTORIZACAO = "
        UPDATE CONSMOVCAIXA SET NRAUTORIZACAO = :VRANTIGO WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";
    
    //Data de vencimento
    const DELETE_ITEMCORVENDA = "
        DELETE FROM ITEMCORVENDA WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA AND NRITEMCORVENDA = :NRITEMCORVENDA
    ";
    
    // Query que busca dados para desconciliar
    const GET_ITEMCORVENDA_TO_CONSMOVCAIXA = "
        SELECT IV.NRITEMCORVENDA, IC.NRITEMCORRECAO, IC.DSITEMCORRECAO, IV.NRCONSMOVCAIXA, TRECEPRODADQ.CDTIPORECE AS TIPORECEANTIGO, IV.VRANTIGO, TRECEPRODADQNOVO.CDTIPORECE AS TIPORECENOVO, IV.VRNOVO
          FROM ITEMCORVENDA IV
          JOIN ITEMCORRECAO IC
            ON IV.NRITEMCORRECAO = IC.NRITEMCORRECAO
          LEFT JOIN PRODUTOADQ PADQ
            ON IV.VRANTIGO = PADQ.NMPRODUTOADQ
          LEFT JOIN PRODUTOADQ PADQNOVO
            ON IV.VRNOVO = PADQNOVO.NMPRODUTOADQ
          LEFT JOIN TPRECEPRODADQ TRECEPRODADQ
            ON PADQ.NRPRODUTOADQ = TRECEPRODADQ.NRPRODUTOADQ
          LEFT JOIN TPRECEPRODADQ TRECEPRODADQNOVO
            ON PADQNOVO.NRPRODUTOADQ = TRECEPRODADQNOVO.NRPRODUTOADQ
         WHERE IV.NRCONSMOVCAIXA = :NRCONSMOVCAIXA
         ORDER BY IV.NRITEMCORVENDA
    ";	
    
    // Deleta Divconciliacao
    const DELETE_DIVCONCILIACAO = "
        DELETE FROM DIVCONCILIACAO
        WHERE NRCONSMOVCAIXA = :NRCONSMOVCAIXA
    ";	
    
    
    // Query que busca dados para deletar financeiro gerado pelo CNC
    const GET_NOTA_BOLETO_CONTA_INFO = "
        SELECT C.NRLANBOLREC, C.NRLANCTONF, C.CDFILIAL, C.NRORG, C.VRATUAREC
        FROM CONTARECEBER C
        JOIN NOTAFISCAL N
          ON C.NRLANCTONF = N.NRLANCTONF
         AND C.CDFILIAL = N.CDFILIAL
		 WHERE N.IDSISTEMA = 'CNC'
		 AND C.CDFILIAL = :CDFILIAL
           AND C.NRLANCTONF = :NRLANCTONF
    ";
    
    
    // VERIFICA QUANTAS CONTAS ESTAO ASSOCIADOS AO BOLETO EM QUESTAO
    const VERIFY_BOLET_CONTA = "
        SELECT :CDFILIAL AS CDFILIAL, :NRLANBOLREC AS NRLANBOLREC, COUNT(C.NRLANCTONF) AS TOT, SUM(VRATUAREC) AS TOTBOLET
          FROM CONTARECEBER C
          JOIN NOTAFISCAL N
            ON C.NRLANCTONF = N.NRLANCTONF
           AND C.CDFILIAL = N.CDFILIAL
          JOIN CONSMOVCAIXA CM
            ON CM.CDFILIALNF = N.CDFILIAL
           AND CM.NRLANCTONF = N.NRLANCTONF
	     WHERE N.IDSISTEMA = 'CNC'
		   AND C.NRLANBOLREC = :NRLANBOLREC
           AND C.CDFILIBOLREC = :CDFILIAL
         GROUP BY :CDFILIAL, :NRLANBOLREC
    ";	
    
    
    //Ordem de deletar financeiro gerado pelo CNC
    // Deleta tipo de recebimento do boleto #1
    const DELETE_TPRECBOLETO_CNC = "
        DELETE FROM TPRECBOLETO
		 WHERE CDFILIAL = :CDFILIAL
           AND NRLANBOLREC = :NRLANBOLREC
    ";	
    
    // EM CASO DE MAIS DE UMA CONTA ASSOCIADA AO BOLETO, NAO DELETAMOS CORRIGIMOS OS VALORES SEM A CONTA QUE ESTA SENDO EDITADA.
    const UPDATE_TPRECBOLETO_CNC = "
        UPDATE TPRECBOLETO
           SET VRTPRECBOLR =  :TOTBOLET
		 WHERE CDFILIAL = :CDFILIAL
           AND NRLANBOLREC = :NRLANBOLREC
    ";	
	
	// Retira o boleto da conta #2
	const UPDATA_CONTARECEBER_CNC = "
		UPDATE CONTARECEBER SET NRLANBOLREC = NULL, CDFILIBOLREC = NULL
		 WHERE CDFILIAL = :CDFILIAL	
           AND NRLANCTONF = :NRLANCTONF
    ";	
	
	// Deleta Boleto a receber #3
	const DELETE_BOLETRECEBER_CNC = "
		DELETE FROM BOLETRECEBER
		 WHERE CDFILIAL = :CDFILIAL
           AND NRLANBOLREC = :NRLANBOLREC
    ";	
	
	// Updata o boleto sem o valor da conta que esta sendo editada
	const UPDATE_BOLETRECEBER_CNC = "
		UPDATE BOLETRECEBER
		   SET VRBOLREC = :TOTBOLET, VRFINALBREC = :TOTBOLET
		 WHERE CDFILIAL = :CDFILIAL
           AND NRLANBOLREC = :NRLANBOLREC
    ";	
	
	// Deleta ajuste de conta a receber #4
	const DELETE_AJCTRECEBER_CNC = "
		DELETE FROM AJCTRECEBER
		 WHERE CDFILIAL = :CDFILIAL
           AND NRLANCTONF = :NRLANCTONF
    ";
	
	// Deleta Conta #5
	const DELETE_CONTARECEBER_CNC = "
		DELETE FROM CONTARECEBER
		 WHERE CDFILIAL = :CDFILIAL	
           AND NRLANCTONF = :NRLANCTONF
    ";
    
    // Deleta nota fiscal #6
    const DELETE_NOTAFISCAL_CNC = "
		DELETE FROM NOTAFISCAL
		 WHERE CDFILIAL = :CDFILIAL
           AND NRLANCTONF = :NRLANCTONF
    ";
}